@@ -21,6 +21,35 @@
 
 G_BEGIN_DECLS
 
+/* repetitive code whose forgetting causes memory leaks */
+static inline gboolean
+mousepad_action_get_state_boolean (GAction *action)
+{
+  GVariant *variant;
+  gboolean  value;
+
+  variant = g_action_get_state (action);
+  value = g_variant_get_boolean (variant);
+  g_variant_unref (variant);
+
+  return value;
+}
+
+/* same, but specific to non-toggle actions with boolean state: an int32 state
+ * is used so that the action is not interpreted as toggle */
+static inline gboolean
+mousepad_action_get_state_int32_boolean (GAction *action)
+{
+  GVariant *variant;
+  gboolean  value;
+
+  variant = g_action_get_state (action);
+  value = g_variant_get_int32 (variant);
+  g_variant_unref (variant);
+
+  return value;
+}
+
 typedef struct _MousepadApplicationClass MousepadApplicationClass;
 typedef struct _MousepadApplication      MousepadApplication;
 
@@ -33,8 +62,9 @@ typedef struct _MousepadApplication      MousepadApplication;
 
 GType                mousepad_application_get_type                   (void) G_GNUC_CONST;
 
-void                 mousepad_application_show_preferences           (MousepadApplication  *application,
-                                                                      GtkWindow            *transient_for);
+GList               *mousepad_application_get_providers              (MousepadApplication  *application);
+
+GtkWidget           *mousepad_application_get_prefs_dialog           (MousepadApplication  *application);
 
 G_END_DECLS
 
