@@ -27,7 +27,8 @@
 
 static void      mousepad_document_finalize                (GObject                *object);
 static void      mousepad_document_notify_cursor_position  (MousepadDocument       *document);
-static void      mousepad_document_notify_encoding         (MousepadFile           *file,
+static void      mousepad_document_delete_range            (MousepadDocument       *document);
+static void      mousepad_document_encoding_changed        (MousepadFile           *file,
                                                             MousepadEncoding        encoding,
                                                             MousepadDocument       *document);
 static void      mousepad_document_notify_language         (GtkSourceBuffer        *buffer,
@@ -38,7 +39,7 @@ static void      mousepad_document_notify_overwrite        (GtkTextView
                                                             MousepadDocument       *document);
 static void      mousepad_document_location_changed        (MousepadDocument       *document,
                                                             GFile                  *file);
-static void      mousepad_document_label_color             (MousepadDocument       *document);
+static void      mousepad_document_style_label             (MousepadDocument       *document);
 static void      mousepad_document_tab_button_clicked      (GtkWidget              *widget,
                                                             MousepadDocument       *document);
 static void      mousepad_document_search_completed        (GObject                *object,
@@ -80,10 +81,9 @@ struct _MousepadDocumentPrivate
 {
   GtkScrolledWindow      __parent__;
 
-  /* the tab label, ebox and CSS provider */
+  /* the tab label and its tooltip ebox */
   GtkWidget              *ebox;
   GtkWidget              *label;
-  GtkCssProvider         *css_provider;
 
   /* utf-8 valid document names */
   gchar                  *utf8_filename;
@@ -218,7 +218,6 @@ mousepad_document_init (MousepadDocument *document)
   document->priv->utf8_filename = NULL;
   document->priv->utf8_basename = NULL;
   document->priv->label = NULL;
-  document->priv->css_provider = gtk_css_provider_new ();
   document->priv->selection_context = NULL;
   document->priv->selection_buffer = NULL;
   document->priv->prev_search_state = INIT;
@@ -266,20 +265,21 @@ mousepad_document_init (MousepadDocument *document)
   gtk_target_list_add_table (target_list, drop_targets, G_N_ELEMENTS (drop_targets));
 
   /* connect handlers to the document attribute signals */
-  g_signal_connect_swapped (document->buffer, "modified-changed",
-                            G_CALLBACK (mousepad_document_label_color), document);
   g_signal_connect_swapped (document->file, "readonly-changed",
-                            G_CALLBACK (mousepad_document_label_color), document);
+                            G_CALLBACK (mousepad_document_style_label), document);
   g_signal_connect_swapped (document->textview, "notify::editable",
-                            G_CALLBACK (mousepad_document_label_color), document);
+                            G_CALLBACK (mousepad_document_style_label), document);
 
   /* forward some document attribute signals more or less directly */
   g_signal_connect_swapped (document->buffer, "notify::cursor-position",
                             G_CALLBACK (mousepad_document_notify_cursor_position), document);
   MOUSEPAD_SETTING_CONNECT_OBJECT (TAB_WIDTH, mousepad_document_notify_cursor_position,
                                    document, G_CONNECT_SWAPPED);
+  g_signal_connect_object (document->buffer, "delete-range",
+                           G_CALLBACK (mousepad_document_delete_range), document,
+                           G_CONNECT_SWAPPED | G_CONNECT_AFTER);
   g_signal_connect (document->file, "encoding-changed",
-                    G_CALLBACK (mousepad_document_notify_encoding), document);
+                    G_CALLBACK (mousepad_document_encoding_changed), document);
   g_signal_connect (document->buffer, "notify::language",
                     G_CALLBACK (mousepad_document_notify_language), document);
   g_signal_connect (document->textview, "notify::overwrite",
@@ -296,7 +296,6 @@ mousepad_document_finalize (GObject *object)
   /* cleanup */
   g_free (document->priv->utf8_filename);
   g_free (document->priv->utf8_basename);
-  g_object_unref (document->priv->css_provider);
 
   /* release the file */
   g_object_unref (document->file);
@@ -343,9 +342,20 @@ mousepad_document_notify_cursor_position (MousepadDocument *document)
 
 
 static void
-mousepad_document_notify_encoding (MousepadFile     *file,
-                                   MousepadEncoding  encoding,
-                                   MousepadDocument *document)
+mousepad_document_delete_range (MousepadDocument *document)
+{
+  g_return_if_fail (MOUSEPAD_IS_DOCUMENT (document));
+
+  if (gtk_text_buffer_get_char_count (document->buffer) == 0)
+    gtk_text_buffer_set_modified (document->buffer, FALSE);
+}
+
+
+
+static void
+mousepad_document_encoding_changed (MousepadFile     *file,
+                                    MousepadEncoding  encoding,
+                                    MousepadDocument *document)
 {
   g_return_if_fail (MOUSEPAD_IS_DOCUMENT (document));
 
@@ -402,8 +412,8 @@ mousepad_document_send_signals (MousepadDocument *document)
   mousepad_document_notify_cursor_position (document);
 
   /* re-send the encoding signal */
-  mousepad_document_notify_encoding (document->file,
-                                     mousepad_file_get_encoding (document->file), document);
+  mousepad_document_encoding_changed (document->file,
+                                      mousepad_file_get_encoding (document->file), document);
 
   /* re-send the language signal */
   mousepad_document_notify_language (GTK_SOURCE_BUFFER (document->buffer), NULL, document);
@@ -425,6 +435,9 @@ mousepad_document_location_changed (MousepadDocument *document,
   g_return_if_fail (MOUSEPAD_IS_DOCUMENT (document));
   g_return_if_fail (file != NULL);
 
+  /* now that there's a backing file, treat empty buffers normally */
+  mousepad_disconnect_by_func (document->buffer, mousepad_document_delete_range, document);
+
   /* get the display filename */
   utf8_filename = mousepad_util_get_display_path (file);
 
@@ -457,15 +470,15 @@ mousepad_document_location_changed (MousepadDocument *document,
       /* set the tab tooltip */
       gtk_widget_set_tooltip_text (document->priv->ebox, utf8_filename);
 
-      /* update label color */
-      mousepad_document_label_color (document);
+      /* update label style */
+      mousepad_document_style_label (document);
     }
 }
 
 
 
 static void
-mousepad_document_label_color (MousepadDocument *document)
+mousepad_document_style_label (MousepadDocument *document)
 {
   GtkStyleContext *context;
 
@@ -483,17 +496,6 @@ mousepad_document_label_color (MousepadDocument *document)
         gtk_style_context_add_class (context, GTK_STYLE_CLASS_DIM_LABEL);
       else
         gtk_style_context_remove_class (context, GTK_STYLE_CLASS_DIM_LABEL);
-
-      /* change the label text color */
-      if (gtk_text_buffer_get_modified (document->buffer))
-        {
-          gtk_css_provider_load_from_data (document->priv->css_provider,
-                                           "label { color: red; }", -1, NULL);
-          gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (document->priv->css_provider),
-                                          GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-        }
-      else
-        gtk_style_context_remove_provider (context, GTK_STYLE_PROVIDER (document->priv->css_provider));
     }
 }
 
@@ -559,11 +561,11 @@ mousepad_document_get_tab_label (MousepadDocument *document)
   gtk_container_add (GTK_CONTAINER (document->priv->ebox), document->priv->label);
   gtk_widget_show (document->priv->label);
 
-  /* set label color */
-  mousepad_document_label_color (document);
+  /* set label style */
+  mousepad_document_style_label (document);
 
   /* create the button */
-  button = mousepad_close_button_new ();
+  button = mousepad_close_button_new (document->buffer);
   gtk_widget_show (button);
 
   /* pack button, add signal and tooltip */
