@@ -772,14 +772,16 @@ mousepad_file_open (MousepadFile  *file,
                       /* we've found a valid bom at the start of the contents */
                       file->write_bom = TRUE;
 
-                      /* advance the contents offset and decrease size */
-                      temp = g_strdup (contents + bom_length);
+                      /* advance the contents offset and decrease size: don't use GLib string
+                       * functions here, there may be null bytes */
+                      file_size -= bom_length;
+                      temp = g_memdup (contents + bom_length, file_size);
                       g_free (contents);
                       contents = temp;
-                      file_size -= bom_length;
 
                       /* set the detected encoding */
                       file->encoding = bom_encoding;
+                      charset = bom_charset;
                     }
                 }
             }
@@ -1009,36 +1011,6 @@ mousepad_file_prepare_save_contents (MousepadFile  *file,
   contents = gtk_text_buffer_get_slice (file->buffer, &start, &end, TRUE);
   length = strlen (contents);
 
-  /* convert to the encoding if different from UTF-8 */
-  if (file->encoding != MOUSEPAD_ENCODING_UTF_8)
-    {
-      /* get the charset */
-      charset = mousepad_encoding_get_charset (file->encoding);
-      if (G_UNLIKELY (charset == NULL))
-        {
-          g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_NO_CONVERSION,
-                       MOUSEPAD_MESSAGE_UNSUPPORTED_ENCODING);
-          g_free (contents);
-
-          return FALSE;
-        }
-
-      /* convert the content to the user encoding */
-      encoded = g_convert (contents, length, charset, "UTF-8", NULL, &written, error);
-
-      /* return if nothing was encoded */
-      if (G_UNLIKELY (encoded == NULL))
-        {
-          g_free (contents);
-          return FALSE;
-        }
-
-      /* set the new contents */
-      g_free (contents);
-      contents = encoded;
-      length = written;
-    }
-
   /* handle line endings */
   if (file->line_ending == MOUSEPAD_EOL_MAC)
     {
@@ -1101,6 +1073,36 @@ mousepad_file_prepare_save_contents (MousepadFile  *file,
       contents[length] = '\0';
     }
 
+  /* convert to the encoding if different from UTF-8: must be done after the above */
+  if (file->encoding != MOUSEPAD_ENCODING_UTF_8)
+    {
+      /* get the charset */
+      charset = mousepad_encoding_get_charset (file->encoding);
+      if (G_UNLIKELY (charset == NULL))
+        {
+          g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_NO_CONVERSION,
+                       MOUSEPAD_MESSAGE_UNSUPPORTED_ENCODING);
+          g_free (contents);
+
+          return FALSE;
+        }
+
+      /* convert the content to the user encoding */
+      encoded = g_convert (contents, length, charset, "UTF-8", NULL, &written, error);
+
+      /* return if nothing was encoded */
+      if (G_UNLIKELY (encoded == NULL))
+        {
+          g_free (contents);
+          return FALSE;
+        }
+
+      /* set the new contents */
+      g_free (contents);
+      contents = encoded;
+      length = written;
+    }
+
   /* add a bom at the start of the contents if needed */
   if (file->write_bom)
     mousepad_encoding_write_bom (&(file->encoding), &length, &contents);
