@@ -54,13 +54,21 @@ typedef enum
 }
 MousepadLineEnding;
 
+/* location type */
+enum
+{
+  MOUSEPAD_LOCATION_VIRTUAL,
+  MOUSEPAD_LOCATION_REVERT,
+  MOUSEPAD_LOCATION_REAL
+};
+
 GType               mousepad_file_get_type                 (void) G_GNUC_CONST;
 
 MousepadFile       *mousepad_file_new                      (GtkTextBuffer       *buffer);
 
 void                mousepad_file_set_location             (MousepadFile        *file,
                                                             GFile               *location,
-                                                            gboolean             real);
+                                                            gint                 type);
 
 GFile              *mousepad_file_get_location             (MousepadFile        *file);
 
