@@ -45,6 +45,7 @@
 
 /* File page */
 #define WID_RECENT_SPIN                     "/prefs/file/history/recent-spin"
+#define WID_SEARCH_SPIN                     "/prefs/file/history/search-spin"
 #define WID_SESSION_COMBO                   "/prefs/file/history/session-combo"
 #define WID_ENCODING_COMBO                  "/prefs/file/history/encoding-combo"
 #define WID_ENCODING_MODEL                  "/prefs/file/history/encoding-model"
@@ -141,6 +142,84 @@ mousepad_prefs_dialog_remove_setting_box (gpointer popover)
 
 
 
+#if defined (GDK_WINDOWING_X11) && ! GTK_CHECK_VERSION (4, 0, 0)
+/*
+ * Popovers suffer from a limitation in GTK 3 on X11: they cannot extend outside their
+ * toplevel window. So we have to resize the dialog before showing the popover in this case.
+ * See https://gitlab.gnome.org/GNOME/gtk/-/issues/543
+ */
+static gboolean
+mousepad_prefs_dialog_drawn (GtkWidget *dialog,
+                             cairo_t   *cr,
+                             gpointer   popover)
+{
+  /* disconnect this handler */
+  mousepad_disconnect_by_func (dialog, mousepad_prefs_dialog_drawn, popover);
+
+  /* it's finally time to show the popover */
+  gtk_widget_show (popover);
+
+  return FALSE;
+}
+
+
+
+static gboolean
+mousepad_prefs_dialog_popover_allocate_finish (gpointer popover)
+{
+  GtkWidget    *dialog, *button;
+  GtkAllocation balloc, dalloc, palloc;
+
+  /* get widget allocations */
+  button = gtk_popover_get_relative_to (popover);
+  dialog = gtk_widget_get_ancestor (button, MOUSEPAD_TYPE_PREFS_DIALOG);
+  gtk_widget_get_allocation (button, &balloc);
+  gtk_widget_get_allocation (dialog, &dalloc);
+  gtk_widget_get_allocation (popover, &palloc);
+
+  /* resize the dialog so it contains the popover */
+  gtk_popover_set_position (GTK_POPOVER (popover), GTK_POS_LEFT);
+  gtk_window_resize (GTK_WINDOW (dialog),
+                     MAX (dalloc.width, palloc.width + dalloc.width - balloc.x),
+                     MAX (dalloc.height, palloc.height));
+
+  /* we will show the popover when the dialog is effectively resized */
+  g_signal_connect_after (dialog, "draw", G_CALLBACK (mousepad_prefs_dialog_drawn), popover);
+
+  return FALSE;
+}
+
+
+
+static void
+mousepad_prefs_dialog_popover_allocate (GtkWidget    *popover,
+                                        GdkRectangle *palloc,
+                                        GtkWidget    *dialog)
+{
+  GtkAllocation dalloc;
+  gint          dx, dy, px, py;
+
+  /* disconnect this handler */
+  mousepad_disconnect_by_func (popover, mousepad_prefs_dialog_popover_allocate, dialog);
+
+  /* do nothing if the dialog already contains the popover */
+  gdk_window_get_origin (gtk_widget_get_window (dialog), &dx, &dy);
+  gdk_window_get_origin (gtk_widget_get_window (popover), &px, &py);
+  gtk_widget_get_allocation (dialog, &dalloc);
+  if (px >= dx && px + palloc->width <= dx + dalloc.width
+      && py >= dy && py + palloc->height <= dy + dalloc.height)
+    return;
+
+  /* we will show the popover when the dialog is resized */
+  gtk_widget_hide (popover);
+
+  /* we need first to exit this handler */
+  g_idle_add_full (G_PRIORITY_HIGH, mousepad_prefs_dialog_popover_allocate_finish, popover, NULL);
+}
+#endif
+
+
+
 static gboolean
 mousepad_prefs_dialog_checkbox_toggled_idle (gpointer data)
 {
@@ -152,9 +231,9 @@ mousepad_prefs_dialog_checkbox_toggled_idle (gpointer data)
   box = mousepad_plugin_provider_get_setting_box (provider);
   visible = gtk_widget_get_visible (button);
 
-  /* the plugin has a setting box and the prefs button is hidden: it is time to show
-   * it with its popover */
-  if (box != NULL && ! visible)
+  /* the plugin has a setting box which is not already packed (e.g. in a dialog)
+   * and the prefs button is hidden: it is time to show it with its popover */
+  if (box != NULL && ! visible && gtk_widget_get_parent (box) == NULL)
     {
       popover = gtk_popover_new (button);
       gtk_container_add (GTK_CONTAINER (popover), box);
@@ -162,6 +241,12 @@ mousepad_prefs_dialog_checkbox_toggled_idle (gpointer data)
       g_signal_connect_swapped (button, "destroy",
                                 G_CALLBACK (mousepad_prefs_dialog_remove_setting_box),
                                 popover);
+#if defined (GDK_WINDOWING_X11) && ! GTK_CHECK_VERSION (4, 0, 0)
+      /* see comment at the beginning of the corresponding code section above */
+      g_signal_connect (popover, "size-allocate",
+                        G_CALLBACK (mousepad_prefs_dialog_popover_allocate),
+                        gtk_widget_get_ancestor (button, MOUSEPAD_TYPE_PREFS_DIALOG));
+#endif
       gtk_widget_show (button);
     }
   /* the setting box was destroyed (normally with its plugin): hide the prefs button */
@@ -250,7 +335,7 @@ mousepad_prefs_dialog_plugins_tab (GtkNotebook *notebook,
       module = provider->data;
       str = g_strconcat ("app.", module->name, NULL);
 
-      /* add an accel label to the provider checkbox */
+      /* add an accel label and a tooltip to the provider checkbox */
       child = gtk_accel_label_new (mousepad_plugin_provider_get_label (provider->data));
       gtk_widget_set_hexpand (child, TRUE);
       accels = gtk_application_get_accels_for_action (GTK_APPLICATION (application), str);
@@ -261,6 +346,7 @@ mousepad_prefs_dialog_plugins_tab (GtkNotebook *notebook,
       gtk_accel_label_set_accel (GTK_ACCEL_LABEL (child), key, mods);
       g_strfreev (accels);
       gtk_container_add (GTK_CONTAINER (widget), child);
+      gtk_widget_set_tooltip_text (widget, mousepad_plugin_provider_get_tooltip (provider->data));
 
       /* add a prefs button to the grid */
       child = gtk_button_new_from_icon_name ("preferences-system", GTK_ICON_SIZE_BUTTON);
@@ -578,6 +664,10 @@ mousepad_prefs_dialog_init (MousepadPrefsDialog *self)
                             "value-changed",
                             G_CALLBACK (mousepad_prefs_dialog_recent_spin_changed), self);
 
+  /* bind the search spin button to the setting */
+  MOUSEPAD_SETTING_BIND (SEARCH_HISTORY_SIZE, gtk_builder_get_object (self->builder, WID_SEARCH_SPIN),
+                         "value", G_SETTINGS_BIND_DEFAULT);
+
   /* bind simple spin buttons to the settings */
   MOUSEPAD_SETTING_BIND (RIGHT_MARGIN_POSITION,
                          gtk_builder_get_object (self->builder, WID_RIGHT_MARGIN_SPIN),
