@@ -318,6 +318,29 @@ mousepad_util_entry_error (GtkWidget *widget,
 
 
 
+gchar *
+mousepad_util_get_selection (GtkTextBuffer *buffer)
+{
+  GtkTextIter  start, end;
+  gchar       *selection = NULL, *escaped;
+
+  if (gtk_text_buffer_get_has_selection (buffer))
+    {
+      gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
+      selection = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+      if (MOUSEPAD_SETTING_GET_BOOLEAN (SEARCH_ENABLE_REGEX))
+        {
+          escaped = g_regex_escape_string (selection, -1);
+          g_free (selection);
+          selection = escaped;
+        }
+    }
+
+  return selection;
+}
+
+
+
 void
 mousepad_util_dialog_create_header (GtkDialog   *dialog,
                                     const gchar *title,
@@ -432,15 +455,33 @@ mousepad_util_decoration_layout_changed (GObject    *settings,
                                          GParamSpec *pspec,
                                          gpointer    bar)
 {
-  gchar *layout, *str;
+  gchar *layout, *p, *str;
 
   g_object_get (settings, "gtk-decoration-layout", &layout, NULL);
 
-  /* add the icon on the far left of the header bar if not already present */
+  /* add the icon on the right of the left buttons if not already present */
   if (g_strstr_len (layout, -1, "icon") == NULL)
     {
-      str = g_strstr_len (layout, -1, ":") == NULL ? ":" : ",";
-      str = g_strconcat ("icon", str, layout, NULL);
+      p = g_strstr_len (layout, -1, ":");
+      if (p == NULL)
+        {
+          g_warn_if_reached ();
+          g_free (layout);
+
+          return;
+        }
+
+      /* whether or not there are buttons on the left */
+      if (p != layout)
+        {
+          str = g_strndup (layout, p - layout);
+          p = g_strconcat (str, ",icon", p, NULL);
+          g_free (str);
+          str = p;
+        }
+      else
+        str = g_strconcat ("icon", layout, NULL);
+
       g_free (layout);
       layout = str;
     }
@@ -470,7 +511,8 @@ mousepad_util_set_titlebar (GtkWindow *window)
 
   /* exit if the user doesn't want client-side decorations, ensuring we have at least
    * one close button or similar in the title bar for all dialogs */
-  if (! MOUSEPAD_SETTING_GET_BOOLEAN (CLIENT_SIDE_DECORATIONS))
+  if (! MOUSEPAD_SETTING_GET_BOOLEAN (CLIENT_SIDE_DECORATIONS)
+      && g_strcmp0 (g_getenv ("GTK_CSD"), "1") != 0)
     {
       if (! GTK_IS_HEADER_BAR (gtk_window_get_titlebar (window)))
         gtk_window_set_titlebar (window, NULL);
@@ -507,7 +549,7 @@ mousepad_util_set_titlebar (GtkWindow *window)
                                    G_CALLBACK (mousepad_util_decoration_layout_changed), bar, 0);
         }
       else
-        gtk_header_bar_set_decoration_layout (GTK_HEADER_BAR (bar), "icon,menu:minimize,maximize,close");
+        gtk_header_bar_set_decoration_layout (GTK_HEADER_BAR (bar), "menu,icon:minimize,maximize,close");
     }
   else
     mousepad_util_decoration_layout_changed (G_OBJECT (settings), NULL, bar);
@@ -1165,6 +1207,40 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
 
 
+gboolean
+mousepad_util_query_exists (GFile    *file,
+                            gboolean  follow_symlink)
+{
+  const gchar *path;
+
+  if (follow_symlink && (path = mousepad_util_get_path (file)) != NULL)
+    return g_file_test (path, G_FILE_TEST_EXISTS);
+  else
+    return g_file_query_exists (file, NULL);
+}
+
+
+
+gchar *
+mousepad_util_get_display_path (GFile *file)
+{
+  gchar *path, *display_path;
+
+  if ((path = g_file_get_path (file)) == NULL)
+    {
+      path = g_file_get_uri (file);
+      display_path = g_uri_unescape_string (path, NULL);
+    }
+  else
+    display_path = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+
+  g_free (path);
+
+  return display_path;
+}
+
+
+
 static void
 mousepad_util_source_remove_all (gpointer  data,
                                  GObject  *object)
